//
//  KJDrawView.swift
//  13-绘图
//
//  Created by apple on 2017/9/11.
//  Copyright © 2017年 HEJJY. All rights reserved.
//

import UIKit

let KMargin : CGFloat = 15
let KStartX : CGFloat = 30

let strokeFillColor : UIColor = UIColor(r: 0, g: 160, b: 172)
let whiteAlphaColor : UIColor = UIColor(r: 255, g: 255, b: 255, alpha: 0.1)

class KJDrawView: UIView {
    
    var xWidth : CGFloat = 0
    
    var drawInfos : [KJDrawInfo] = [KJDrawInfo]()
    var pointArray : [CGPoint] = [CGPoint]()
    
    init(frame: CGRect, drawInfos : [KJDrawInfo], xWidth : CGFloat) {
        super.init(frame: frame)
        
        backgroundColor = UIColor.clear
        self.xWidth = xWidth
        self.drawInfos = drawInfos
        
        // 转换坐标
        pointArray = convertToPoints(drawInfos: drawInfos)
        
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    
    // Only override draw() if you perform custom drawing.
    // An empty implementation adversely affects performance during animation.
    override func draw(_ rect: CGRect) {
        
        // 数据为空，直接返回
        if pointArray.count == 0 { return }
        
        // 获取上下文
        let context = UIGraphicsGetCurrentContext()
        
        // 设置绘制和填充颜色
        context?.setStrokeColor(strokeFillColor.cgColor)
        context?.setFillColor(strokeFillColor.cgColor)
        
        // 设置线宽
        context?.setLineWidth(0.5)
        
        // 绘制两条线
        let topPoints : [CGPoint] = [CGPoint(x: KMargin, y: KMargin), CGPoint(x: self.width - KMargin, y: KMargin)]
        context?.addLines(between: topPoints)
        context?.strokePath()
        
        let bottomPoints : [CGPoint] = [CGPoint(x: KMargin, y: self.height - KMargin), CGPoint(x: self.width - KMargin, y: self.height - KMargin)]
        context?.addLines(between: bottomPoints)
        context?.strokePath()

    
        // 绘制坐标点
        for i in 0..<pointArray.count {
            
            let point : CGPoint = pointArray[i]
            
            context?.addArc(center: point, radius: 2, startAngle: 0, endAngle: 2 * .pi, clockwise: true)
            context?.fillPath()
        }
        
        let attrs : [String : Any] = [NSFontAttributeName : UIFont.systemFont(ofSize: 9), NSForegroundColorAttributeName : UIColor.gray]
        
        // 绘制横坐标
        for j in 0..<pointArray.count {
            
            let point : CGPoint = pointArray[j]
            let drawInfo : KJDrawInfo = drawInfos[j]
            let dateStr : NSString = drawInfo.date as NSString
            
            dateStr.draw(at: CGPoint(x: point.x - KMargin, y: self.height - KMargin + 2), withAttributes: attrs)
            
        }
        
        // 绘制0分、100分
        let allStr : NSString  = "100分"
        allStr.draw(at: CGPoint(x: KMargin, y: 0), withAttributes: attrs)
        
        let zeroStr : NSString = "0分"
        zeroStr.draw(at: CGPoint(x: KMargin, y: self.height - 2 * KMargin), withAttributes: attrs)
        
        // 绘制最近得分
        let lastStr : NSString = "本次得分:90分"
        lastStr.draw(at: CGPoint(x: self.width - 80, y: 0), withAttributes: attrs)
        
        // 绘制坐标点之间的线
        context?.setLineDash(phase: 0, lengths: [0])// 避免虚化
        let path = CGMutablePath()
        
        for index in 0..<pointArray.count - 1 {
            
            let start : CGPoint = pointArray[index]
            let end : CGPoint = pointArray[index + 1]

            drawLine(path: path, start: start, end: end)
        }
        
        context?.strokePath()
        
        // 绘制渐变区域
        drawGradient()
    }


}

extension KJDrawView {
    
    // 动画绘制
    func drawLine(path : CGMutablePath, start : CGPoint, end : CGPoint) {
        
        let shapeLayer = CAShapeLayer()
        shapeLayer.lineWidth = 0.5
        shapeLayer.lineCap = kCALineCapButt
        shapeLayer.strokeColor = strokeFillColor.cgColor
        shapeLayer.fillColor = strokeFillColor.cgColor
        
        path.move(to: start)
        path.addLine(to: end)
        shapeLayer.path = path
        
        let animation = CABasicAnimation(keyPath: "strokeEnd")
        animation.duration = 1
        animation.timingFunction = CAMediaTimingFunction(name: kCAMediaTimingFunctionEaseOut)
        animation.fromValue = 0
        animation.toValue = 1
        animation.autoreverses = false
        animation.fillMode = kCAFillModeForwards
        shapeLayer.animation(forKey: "strokeEndAnimation")
        
        self.layer.addSublayer(shapeLayer)
        
    }
    
    // 添加渐变色
    func drawGradient() {
        
        let gradientLayer = CAGradientLayer()
        gradientLayer.frame = CGRect(x: 0, y: 0, width: self.width, height: self.height)
        gradientLayer.colors = [strokeFillColor.cgColor, whiteAlphaColor.cgColor]
        gradientLayer.locations = [NSNumber(value: 0), NSNumber(value: 1)]
        gradientLayer.startPoint = CGPoint(x: 0.0, y: 1.0)
        gradientLayer.endPoint = CGPoint(x: 0.0, y: 0.0)
        
        // 画一个封闭渐变区域
        let gradientPath = UIBezierPath()
        gradientPath.move(to: CGPoint(x: KStartX, y: self.height - KMargin))
        
        for i in 0..<pointArray.count {
            
            gradientPath.addLine(to: pointArray[i])
        }
        
        let endPoint = pointArray.last
        if let end = endPoint {
            let newEnd = CGPoint(x: end.x, y: self.height - KMargin)
            gradientPath.addLine(to: newEnd)
        }
        
        let shapeLayer = CAShapeLayer()
        shapeLayer.path = gradientPath.cgPath
        gradientLayer.mask = shapeLayer
        self.layer.addSublayer(gradientLayer)
    }

    // 将info数据转化为坐标点
    func convertToPoints(drawInfos : [KJDrawInfo]) -> [CGPoint] {
        
        var pointArray : [CGPoint] = [CGPoint]()
        
        for i in 0..<drawInfos.count {
            let drawInfo : KJDrawInfo = drawInfos[i]
            var point : CGPoint = CGPoint()
            point.x = KStartX + xWidth * CGFloat(i)
            point.y = self.height - (CGFloat(drawInfo.score) / 100 * (self.height - 2 * KMargin) + KMargin)
            pointArray.append(point)
        }
        
        return pointArray
    }
    
}
